@@ -16,16 +16,16 @@
 import abc
 import collections
 import copy
+import enum
 import io
 import json
 import logging
-import re
 import zipfile
 
 import pkg_resources
 
 from sushy import exceptions
-from sushy.resources import mappings as res_maps
+from sushy.resources import constants
 from sushy.resources import oem
 from sushy import utils
 
@@ -269,18 +269,28 @@ class MappedField(Field):
         :param field: JSON field to fetch the value from. This can be either
             a string or a list of string. In the latter case, the value will
             be fetched from a nested object.
-        :param mapping: a mapping to take values from.
+        :param mapping: a mapping to take values from, a dictionary or
+            an enumeration.
         :param required: whether this field is required. Missing required,
             but not defaulted, fields result in MissingAttributeError.
         :param default: the default value to use when the field is missing.
             This value is not matched against the mapping.
         """
-        if not isinstance(mapping, collections.abc.Mapping):
-            raise TypeError("The mapping argument must be a mapping")
+        if isinstance(mapping, type) and issubclass(mapping, enum.Enum):
+            def adapter(value):
+                try:
+                    return mapping(value)
+                except ValueError:
+                    return None
+
+        elif isinstance(mapping, collections.abc.Mapping):
+            adapter = mapping.get
+        else:
+            raise TypeError("The mapping argument must be a mapping or "
+                            "an enumeration")
 
         super(MappedField, self).__init__(
-            field, required=required, default=default,
-            adapter=mapping.get)
+            field, required=required, default=default, adapter=adapter)
 
 
 class MappedListField(Field):
@@ -300,10 +310,20 @@ class MappedListField(Field):
             but not defaulted, fields result in MissingAttributeError.
         :param default: the default value to use when the field is missing.
         """
-        if not isinstance(mapping, collections.abc.Mapping):
-            raise TypeError("The mapping argument must be a mapping")
+        if isinstance(mapping, type) and issubclass(mapping, enum.Enum):
+            def adapter(value):
+                try:
+                    return mapping(value)
+                except ValueError:
+                    return None
 
-        self._mapping_adapter = mapping.get
+        elif isinstance(mapping, collections.abc.Mapping):
+            adapter = mapping.get
+        else:
+            raise TypeError("The mapping argument must be a mapping or "
+                            "an enumeration")
+
+        self._mapping_adapter = adapter
         super(MappedListField, self).__init__(
             field, required=required, default=default,
             adapter=lambda x: x)
@@ -339,8 +359,7 @@ class MessageListField(ListField):
     message = Field('Message')
     """Human readable message, if provided"""
 
-    severity = MappedField('Severity',
-                           res_maps.SEVERITY_VALUE_MAP)
+    severity = MappedField('Severity', constants.Severity)
     """Severity of the error"""
 
     resolution = Field('Resolution')
@@ -494,6 +513,9 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
 
     links = LinksField('Links')
 
+    _log_resource_body = True
+    """Whether to log the whole resource body in debug mode."""
+
     def __init__(self,
                  connector,
                  path='',
@@ -578,21 +600,7 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
 
         :returns ETag or None
         """
-        etag = self._get_headers().get('ETag')
-        if not etag:
-            return None
-
-        # Note (arne_wiebalck): in case there is a weak Etag,
-        # return it with and without the qualifier to handle
-        # vendor implementations which may require one or the
-        # other (but should actually not use weak tags in the
-        # first place).
-        pattern = re.compile(r'^(W\/)("\w*")$')
-        match = pattern.match(etag)
-        if match:
-            return etag + ',' + match.group(2)
-
-        return etag
+        return self._get_headers().get('ETag')
 
     def _get_headers(self):
         """Returns the HTTP headers of the request for the resource.
@@ -643,7 +651,9 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
         attributes = self._parse_attributes(self._json)
         LOG.debug('Received representation of %(type)s %(path)s: %(json)s',
                   {'type': self.__class__.__name__,
-                   'path': self._path, 'json': attributes})
+                   'path': self._path,
+                   'json': (attributes if self._log_resource_body
+                            else '<stripped>')})
         self._do_refresh(force)
 
         # Mark it fresh
@@ -734,14 +744,7 @@ class ResourceBase(object, metaclass=abc.ABCMeta):
         return self._root
 
 
-class ResourceCollectionBase(ResourceBase, metaclass=abc.ABCMeta):
-
-    name = Field('Name')
-    """The name of the collection"""
-
-    members_identities = Field('Members', default=[],
-                               adapter=utils.get_members_identities)
-    """A tuple with the members identities"""
+class ResourceLinksBase(ResourceBase, metaclass=abc.ABCMeta):
 
     def __init__(self, connector, path, redfish_version=None, registries=None,
                  root=None):
@@ -757,12 +760,17 @@ class ResourceCollectionBase(ResourceBase, metaclass=abc.ABCMeta):
             used in any resource that needs registries to parse messages.
         :param root: Sushy root object. Empty for Sushy root itself.
         """
-        super(ResourceCollectionBase, self).__init__(
-            connector, path, redfish_version, registries, root=root)
+        super().__init__(connector, path, redfish_version, registries,
+                         root=root)
         LOG.debug('Received %(count)d member(s) for %(type)s %(path)s',
                   {'count': len(self.members_identities),
                    'type': self.__class__.__name__, 'path': self._path})
 
+    @property
+    @abc.abstractmethod
+    def members_identities(self):
+        """A sequence with members identities"""
+
     @property
     @abc.abstractmethod
     def _resource_type(self):
@@ -791,3 +799,40 @@ class ResourceCollectionBase(ResourceBase, metaclass=abc.ABCMeta):
         :returns: A list of ``_resource_type`` objects
         """
         return [self.get_member(id_) for id_ in self.members_identities]
+
+
+class ResourceCollectionBase(ResourceLinksBase):
+
+    name = Field('Name')
+    """The name of the collection"""
+
+    members_identities = Field('Members', default=[],
+                               adapter=utils.get_members_identities)
+    """A tuple with the members identities"""
+
+
+class MutableResourceCollectionBase(ResourceCollectionBase):
+
+    def _create_member(self, values):
+        """Create a new member of this collection.
+
+        :param values: Fields to set on creation.
+        :return: Created resource or None if it was not returned by the server.
+        """
+        response = self._conn.post(self._path, data=values)
+        self.invalidate(force_refresh=True)
+
+        location = response.headers.get('Location')
+        if location is None:
+            return None
+
+        return self._resource_type(
+            self._conn, location,
+            redfish_version=self.redfish_version,
+            registries=self.registries,
+            root=self.root)
+
+    def delete_member(self, identity):
+        """Delete the given member of the collection."""
+        self._conn.delete(identity)
+        self.invalidate(force_refresh=True)
